######################################################################
#  CALCULATION OF (AGE-YEAR SPECIFIC) DISABILITY INCIDENCE RATES 
#                     DISABILITY RECOVERY RATE
#                   AND MORTALITY RATES FOR 4 CAT AND N YEARS
#
#     BASED ON (AGE-SPECIFIC AND YEAR-SPECIFIC) RELATIVE RISKS
#              TOTAL (AGE-SPECIFIC AND YEAR-SPECIFIC) MORTALITY RATES
#              (AGE-SPECIFIC AND YEAR-SPECIFIC) PREVALENCE
#
#  ASSUME  NO BMI INITIATION (after age 55)
#
########################################################################
rm(list=ls())

source("Mort_ADLinc_BMI_fun.r")
###########################################################

age.start=55
age.end = 90
year.start = 2008
year.end = 2008
n.age = age.end-age.start+1
n.year = year.end-year.start+1

###################################################################
#              INPUT
###################################################################
                    
gender = "M"          # Either M(ale) or F(emale)     

age.start=55
age.end = 100
n.age = age.end-age.start+1
# Relative risk for all countries and ages the same 
if (gender == "F" ) {
   RR.mort <- matrix(c(1.11, 1.55, 1.41), ncol=3, nrow=n.age, byrow=T)
   rr.inc = 1.715  # gewogen gemid uit Walter et al
} 
if (gender == "M" ){
   RR.mort <- matrix(c(1.11, 1.89, 1.72), ncol=3, nrow=n.age, byrow=T)
   rr.inc = 1.715  # gewogen gemid uit Walter et al
}   
RR_OD <- matrix(RR.mort[,1], nrow=n.age)
RR_OnD <- matrix(RR.mort[,2], nrow=n.age)
RR_nOD <- matrix(RR.mort[,3], nrow=n.age)

#name1 = paste(country.name,gender, sep="_")
#risk.name1 =paste(risk.factor,gender, sep="_")

Prev.name = paste("Prev_ADLxBMI_NL",gender,sep="_")
Prev.name = paste(Prev.name,"csv",sep=".")
Mort.name = paste("Mort_NL", gender,sep="_")
Mort.name = paste(Mort.name,"csv",sep=".")
#Disinc.name = paste("Disinc_ADL_NL", gender,sep="_")
#Disinc.name = paste(Disinc.name,"txt",sep=".")

TotRates <- read.table(file = Mort.name,header=T, sep=",", 
                     colClasses ="numeric")
#DisincRates <- read.table(file = Disinc.name,header=T, sep=";", 
#                     colClasses ="numeric")
Prev <- read.csv(file = Prev.name, header=T, sep=",", 
                     colClasses ="numeric")

Prev_OD <- matrix(rep(Prev[Prev[,1] %in% age.start:age.end,"OD"],1), nrow=n.age)
Prev_OnD <- matrix(rep(Prev[Prev[,1] %in% age.start:age.end,"OnD"],1), nrow=n.age)
Prev_nOD <- matrix(rep(Prev[Prev[,1] %in% age.start:age.end,"nOD"],1), nrow=n.age)
Prev_nOnD <- matrix(rep(Prev[Prev[,1] %in% age.start:age.end,"nOnD"],1), nrow=n.age)

############################################################
# 1)   FIRST STEP CALCULATE THE MORTALITY 
#           START VALUES FOR SIMULATENOUS ESTIMATION
###########################################################
m_OD = matrix(0,nrow=n.age,ncol=n.year)
m_OnD = matrix(0,nrow=n.age,ncol=n.year)
m_nOD = matrix(0,nrow=n.age,ncol=n.year)
m_nOnD = matrix(0,nrow=n.age,ncol=n.year)
rownames(m_OD) = age.start:age.end
rownames(m_OnD) = age.start:age.end
rownames(m_nOD) = age.start:age.end
rownames(m_nOnD) = age.start:age.end
colnames(m_OD) = year.start:year.end
colnames(m_OnD) = year.start:year.end
colnames(m_nOD) = year.start:year.end
colnames(m_nOnD) = year.start:year.end

age.start1 = 55
for (year in year.start:year.end) {
 for (age in age.start1:age.end) {
    i = age-age.start+1
    j = year-year.start+1
    prev = c(Prev_OD[i,j],Prev_OnD[i,j],Prev_nOD[i,j],Prev_nOnD[i,j])
    relrisk = c(RR_OD[i,j],RR_OnD[i,j], RR_nOD[i,j])
    r0 = start.rates(prev,relrisk,TotRates[i,j])
    r1 = r0 - calratesS.fun(r0,prev,relrisk,TotRates[i,j])/
              calratesS.dfun(r0,prev,relrisk,TotRates[i,j])
    while (abs(r0-r1) > 1e-6) {
       r0=r1
       r1 = r0 - calratesS.fun(r0,prev,relrisk,TotRates[i,j])/
              calratesS.dfun(r0,prev,relrisk,TotRates[i,j])
    }
    m_OD[i,j]=relrisk[1]*r1 
    m_OnD[i,j]=relrisk[2]*r1 
    m_nOD[i,j]=relrisk[3]*r1 
    m_nOnD[i,j]=r1    
 }
}  

#######################################################################
# 2)   SECOND STEP CALCULATION OF THE INCIDENCE RATES FOR EACH GROUP
#           START VALUES FOR SIMULATENOUS ESTIMATION
# from DISABILITY MODEL, i.e. (age-year specific) Markov model 
# with rate matrix  (conditional on BMI status)
#
#          Dis         non-Dis   death
#  M = -(phi+m_D)       phi      m_D
#         theta    -(theta+m_N)  m_N
#          0            0         0
#
#################################################################
ind.BMI = c(0,1)
theta= array(0,dim=c(n.age-1,n.year,2))
rownames(theta) = age.start:(age.end-1)
colnames(theta) = year.start:year.end
dimnames(theta)[[3]] = c("theta_O", "theta_nO")
TP= array(0,dim=c(2,2,n.age-1,2))
rownames(TP) = c("D", "nD")
colnames(TP) = c("D", "nD")
dimnames(TP)[[3]] = age.start:(age.end-1)
dimnames(TP)[[4]] = c("Obese", "non-Obese")
TP.eig= array(0,dim=c(3,3,n.age-1,2))
rownames(TP.eig) = c("D", "nD", "dead")
colnames(TP.eig) = c("D", "nD", "dead")
dimnames(TP.eig)[[3]] = age.start:(age.end-1)
dimnames(TP.eig)[[4]] = c("Obese", "non-Obese")
for (year in year.start:year.end) {
 for (age in age.start1:(age.end-1)) {
    i = age-age.start+1
    j = year-year.start+1      
    for (s in 1:2){                  
      propD0 = ifelse(ind.BMI[s]==1, Prev_OD[i,j]/(Prev_OD[i,j]+Prev_OnD[i,j]), 
                                     Prev_nOD[i,j]/(Prev_nOD[i,j]+Prev_nOnD[i,j]))
      propD1 = ifelse(ind.BMI[s]==1, Prev_OD[i+1,j]/(Prev_OD[i+1,j]+Prev_OnD[i+1,j]), 
                                      Prev_nOD[i+1,j]/(Prev_nOD[i+1,j]+Prev_nOnD[i+1,j]))
      mDis = ifelse(ind.BMI[s]==1, m_OD[i,j], m_nOD[i,j])
      mnDis = ifelse(ind.BMI[s]==1, m_OnD[i,j], m_nOnD[i,j])
      theta0 = start.inc.rates(1-propD0,1-propD1)
      theta0=theta0*(theta0 > 0) + (theta0 <=0)*0
      theta1 = theta0 - fun.inc(theta0, propD0, propD1, mDis,mnDis,phi=0)/
                        dfun.inc(theta0, propD0, propD1, mDis,mnDis,phi=0)
      while (abs(theta0-theta1) > 1e-6) {
          theta0 = theta1
          theta1 = theta0 - fun.inc(theta0, propD0, propD1, mDis,mnDis,phi=0)/
                           dfun.inc(theta0, propD0, propD1, mDis,mnDis,phi=0)
      }
      theta[i,j,s]=theta0*(theta0 > 0) + (theta0 <=0)*0
      TP[,,i,s] = transprob(theta[i,j,s],phi=0,mD=mDis,mnD=mnDis)
      TP.eig[,,i,s] = Transprob.eig(theta[i,j,s],phi=0,mD=mDis,mnD=mnDis)
    }  
 }
}
#### NO recovery #### two theta and no  recovery ####
theta00= array(0,dim=c(n.age-1,n.year,2))
rownames(theta00) = age.start:(age.end-1)
colnames(theta00) = year.start:year.end
dimnames(theta00)[[3]] = c("theta_O", "theta_nO")
mnOnD.mat = matrix(0,nrow= n.age,ncol=n.year)
rownames(mnOnD.mat) = age.start:(age.end)
colnames(mnOnD.mat) = year.start:year.end
pars.mat = array(0,dim=c(n.age-1,n.year,2))
rownames(pars.mat) = age.start:(age.end-1)
colnames(pars.mat) = year.start:year.end
#dimnames(pars.mat)[[3]]=c("theta_nO", "theta_O")
dimnames(pars.mat)[[3]]=c("theta_O", "theta_nO")
for (year in year.start:year.end) {
 for (age in age.start:(age.end-1)) {
    i = age-age.start+1
    j = year-year.start+1
    mnOnD.mat[i,j] = m_nOnD[i,j]
    pars.mat[i,j,] = theta[i,j,]
 }
}     

Surv.mat = array(0,dim=c(n.age-1,n.year,3))
rownames(Surv.mat) = age.start:(age.end-1)
colnames(Surv.mat) = year.start:year.end
dimnames(Surv.mat)[[3]] = c("tot", "first", "opt")

for (year in year.start:year.end) {
 for (age in age.start:(age.end-1)) {
    i = age-age.start+1
    j = year-year.start+1
    mnOnD0 = mnOnD.mat[i,j]
    # optimization over mnOnD
    prop0m = c(Prev_OD[i,j],Prev_OnD[i,j],Prev_nOD[i,j],Prev_nOnD[i,j])
    names(prop0m)=c("OD","OnD","nOD","nOnD")
    relrisk = c(RR_OD[i,j],RR_OnD[i,j], RR_nOD[i,j])
    theta1a = pars.mat[i,j,]
     
    optm=optimize(f=surv.inc2.O,interval=c(0,1),theta=theta1a,prop0=prop0m,relrisk=relrisk, 
                mtot=TotRates[i,j],phi=0 ) 
    mnOnD1 = optm$minimum
    mnOnD.mat[i,j] = mnOnD1
 
    # optimization over theta
     mort.rates = matrix(c(relrisk[1]*mnOnD1,relrisk[3]*mnOnD1, 
                          relrisk[2]*mnOnD1,mnOnD1), nrow=2)
    for (s in 1:2){                  
      propD0 = ifelse(ind.BMI[s]==1, Prev_OD[i,j]/(Prev_OD[i,j]+Prev_OnD[i,j]), 
                                     Prev_nOD[i,j]/(Prev_nOD[i,j]+Prev_nOnD[i,j]))
      propD1 = ifelse(ind.BMI[s]==1, Prev_OD[i+1,j]/(Prev_OD[i+1,j]+Prev_OnD[i+1,j]), 
                                      Prev_nOD[i+1,j]/(Prev_nOD[i+1,j]+Prev_nOnD[i+1,j]))
      mDis = ifelse(ind.BMI[s]==1, mort.rates[1,1], mort.rates[2,1])
      mnDis = ifelse(ind.BMI[s]==1, mort.rates[1,2], mort.rates[2,2])
      theta0 = theta1a[s]
      theta00[i,j,s]=theta0
      theta1 = theta0 - fun.inc(theta0, propD0, propD1, mDis,mnDis,phi=0)/
                        dfun.inc(theta0, propD0, propD1, mDis,mnDis,phi=0)
      while (abs(theta0-theta1) > 1e-6) {
          theta0 = theta1
          theta1 = theta0 - fun.inc(theta0, propD0, propD1, mDis,mnDis,phi=0)/
                           dfun.inc(theta0, propD0, propD1, mDis,mnDis,phi=0)
      }
      pars.mat[i,j,s]=theta0*(theta0 > 0) + (theta0 <=0)*0
    }  
   while (abs(mnOnD0-mnOnD1) > 1e-10) {
      mnOnD0 = mnOnD1
      # optimization over mnOnD
      theta1a = pars.mat[i,j,]
      optm=optimize(f= surv.inc2.O,interval=c(0,1),theta=theta1a,prop0=prop0m,relrisk=relrisk,
                mtot=TotRates[i,j], phi=0 ) 
      mnOnD1 = optm$minimum
        
      # optimization over theta
      mort.rates = matrix(c(relrisk[1]*mnOnD1,relrisk[3]*mnOnD1, 
                          relrisk[2]*mnOnD1,mnOnD1), nrow=2)
      for (s in 1:2){                  
       propD0 = ifelse(ind.BMI[s]==1, Prev_OD[i,j]/(Prev_OD[i,j]+Prev_OnD[i,j]), 
                                     Prev_nOD[i,j]/(Prev_nOD[i,j]+Prev_nOnD[i,j]))
       propD1 = ifelse(ind.BMI[s]==1, Prev_OD[i+1,j]/(Prev_OD[i+1,j]+Prev_OnD[i+1,j]), 
                                      Prev_nOD[i+1,j]/(Prev_nOD[i+1,j]+Prev_nOnD[i+1,j]))
        mDis = ifelse(ind.BMI[s]==1, mort.rates[1,1], mort.rates[2,1])
        mnDis = ifelse(ind.BMI[s]==1, mort.rates[1,2], mort.rates[2,2])
        theta0 = theta1a[s]
        theta00[i,j,s]=theta0
        theta1 = theta0 - fun.inc(theta0, propD0, propD1, mDis,mnDis,phi=0)/
                        dfun.inc(theta0, propD0, propD1, mDis,mnDis,phi=0)
        while (abs(theta0-theta1) > 1e-6) {
          theta0 = theta1
          theta1 = theta0 - fun.inc(theta0, propD0, propD1, mDis,mnDis,phi=0)/
                           dfun.inc(theta0, propD0, propD1, mDis,mnDis,phi=0)
       }
      pars.mat[i,j,s]=theta0*(theta0 > 0) + (theta0 <=0)*0
    }  
    } 
    mnOnD.mat[i,j] = mnOnD1
    Surv.mat[i,j,1]=exp(-TotRates[i,j])
 } 
 i = age.end-age.start+1
 mnOnD.mat[i,j]=m_nOnD[i,j]
}
mort_1 = cbind(mnOnD.mat*RR_OD, mnOnD.mat*RR_OnD,mnOnD.mat*RR_nOD,mnOnD.mat)
colnames(mort_1)=c("OD","OnD","nOD","nOnD")
rownames(mort_1) = age.start:age.end
##################################################################################
# simultaneous estimation of disability indicence (theta), dep of BMI status
####################################################################################
pars_O= array(0,dim=c(n.age-1,n.year,2))
rownames(pars_O) = age.start:(age.end-1)
colnames(pars_O) = year.start:year.end
dimnames(pars_O)[[3]] = c("theta_O","theta_nO")
m.O = matrix(0,nrow= n.age,ncol=n.year)
rownames(m.O) = age.start:(age.end)
colnames(m.O) = year.start:year.end
fnval= array(0,dim=c(n.age-1,n.year,2))
rownames(fnval) = age.start:(age.end-1)
colnames(fnval) = year.start:year.end
fnval3= array(0,dim=c(n.age-1,n.year,2))
rownames(fnval3) = age.start:(age.end-1)
colnames(fnval3) = year.start:year.end

pars_O = pars.mat
m.O = m_nOnD

year=2008
for (age in age.start:(age.end-1)) {
    i = age-age.start+1
    j = year-year.start+1
    # Optimization over (theta_nO,theta_O)
    prop0 = c(Prev_OD[i,j],Prev_OnD[i,j],Prev_nOD[i,j],Prev_nOnD[i,j])
    prop1 =c(Prev_OD[i+1,j],Prev_OnD[i+1,j],Prev_nOD[i+1,j],Prev_nOnD[i+1,j])
    mort.rates =mort_1[i,]
    
    pars0=pars.mat[i,j,]
    fnval[i,j,1] = quaddiff.O(pars=pars0, prop0,prop1,mort.rates)  
    optpar=optim(pars0, fn=quaddiff.O,Prev0=prop0,Prev1=prop1,m=mort.rates,
                     method = "Nelder-Mead")  
    pars1 = optpar$par
    pars_O[i,j,]=ifelse(pars1>1e-10, t(pars1), 1e-10)
    fnval[i,j,2]=optpar$value
    
    # optimization over mnOnD
    mnOnD0 = mnOnD.mat[i,j]
    relrisk = c(RR_OD[i,j],RR_OnD[i,j], RR_nOD[i,j])
    names(relrisk) = c("OD", "OnD", "nOD")
    theta1a = pars_O[i,j,]
     
    optm=optimize(f=surv.O,interval=c(0,1),th=theta1a,prop0,relrisk=relrisk, 
                mtot=TotRates[i,j] ) 
    mnOnD1 = optm$minimum
    m.O[i,j] = mnOnD1
    mort.rates =c(mnOnD1*RR_OD[i,j], mnOnD1*RR_OnD[i,j],mnOnD1*RR_nOD[i,j],mnOnD1)
    names(mort.rates)=c("OD", "OnD", "nOD", "nOnD")
    while (abs(mnOnD0-mnOnD1) > 1e-10) {
      print(c(i,j))
      mnOnD0 =mnOnD1
      pars0=pars_O[i,j,]
      fnval3[i,j,1] = quaddiff.O(pars=pars0, prop0,prop1,mort.rates)  
      optpar=optim(pars0, fn=quaddiff.O,Prev0=prop0,Prev1=prop1,m=mort.rates,
                     method = "Nelder-Mead")  
      pars1 = optpar$par
      pars_O[i,j,]=ifelse(pars1>1e-10, t(pars1), 1e-10)
      fnval3[i,j,2]=optpar$value
    
      # optimization over mnOnD
      theta1a = pars_O[i,j,]
      optm=optimize(f=surv.O,interval=c(0,1),th=theta1a,prop0,relrisk=relrisk, 
                mtot=TotRates[i,j] ) 
      mnOnD1 = optm$minimum
      m.O[i,j] = mnOnD1
      mort.rates =c(mnOnD1*RR_OD[i,j], mnOnD1*RR_OnD[i,j],mnOnD1*RR_nOD[i,j],mnOnD1)
      names(mort.rates)=c("OD", "OnD", "nOD", "nOnD")
    }
    #c(pars.mat[20,1,],pars_O[20,1,])
    #c(fnval[20,1,],fnval3[20,1,])
}
i = age.end-age.start+1
m.O[i,j]=mnOnD.mat[i,j]
 
cbind(pars.mat[,1,],pars_O[,1,"theta_O"],pars_O[,1,"theta_nO"])
cbind(fnval[,1,],fnval3[,1,1])
cbind(mnOnD.mat[,1],m.O[,1])
mort_2 = cbind(m.O*RR_OD, m.O*RR_OnD,m.O*RR_nOD,m.O)
colnames(mort_2)=c("OD","OnD","nOD","nOnD")
rownames(mort_2) = age.start:age.end

theta1 = pars.mat[,1,]
mort = cbind(m.O, m.O*RR_nOD,m.O*RR_OnD,m.O*RR_OD)
colnames(mort)=c("nOnD","nOD","OnD","OD")
rownames(mort) = age.start:age.end
if(0){ 
 if (gender== "M") {
  # males
  write.table(theta1,"ADLinc2_BMI30_DE_M_8mrt11.txt", sep = ";", quote=F, eol = "\n", col.names=T, row.names=T)
  write.table(mort,"Mort2_BMI30_DE_M_8mrt11.txt", sep = ";", quote=F, eol = "\n", col.names=T, row.names=T)
  }
 if (gender== "F") {
  # females
  write.table(theta1,"ADLinc2_BMI30_DE_F_8mrt11.txt", sep = ";", quote=F, eol = "\n", col.names=T, row.names=T)
  write.table(mort,"Mort2_BMI30_DE_F_8mrt11.txt", sep = ";", quote=F, eol = "\n", col.names=T, row.names=T)
 }
}
